#pragma once

#include <list>
#include <map>
#include <atlbase.h>
#include <atlwin.h>

#include "ipcinterface.h"
namespace tipbase
{
const int MAX_IPC_STRING_NAME = 1024;

class IPCStringKey : public IIPCKey
{
public:
	IPCStringKey( const TCHAR *name );
	void* GetData();
	int GetSize();

private:
	TCHAR	m_name[MAX_IPC_STRING_NAME];
	int		m_name_size;
};

typedef std::list<IPCPacket> IPCPacketQueue;
class IPCCopyDataMessageWindow;
class IPCCopyDataServer;

class IPCCopyDataHandle : public IIPCHandle
{
friend IPCCopyDataServer;
public:
	IPCCopyDataHandle(HWND hWnd, HWND hServerWnd, LPCTSTR name, IPCCopyDataServer* server);
	virtual ~IPCCopyDataHandle();
	bool Read(IPCPacket& packet);
	bool Write( const BYTE *data, int size );
	IIPCKey* GetKey();
	void Close();
	void CloseWithSend();
	void CloseWithoutSend();
	bool IsValid();

private:
	void ReceiveData( const BYTE *data, int size );
	void PushPacketInQueue(IPCPacket& packet);
	void SetContinueMode( int size );
	
private:
	IPCCopyDataServer*	m_server;
	IPCPacketQueue		m_queue;
	IPCStringKey		m_key;
	HWND				m_hWnd;
	HWND				m_serverWnd;
	bool				m_continueMode;
	int					m_continueSize;
	int					m_keepSize;
	BYTE*				m_buffer;
};

const int IPC_COPYDATA_MAX_SEND_SIZE = 255;

BOOL VistaMessagePatch();

class IPCCopyDataServerEventHooker
{
public:
	virtual ~IPCCopyDataServerEventHooker();
	virtual int Connected( IIPCHandle *connectedHandle );
	virtual void Close( IIPCHandle *connectedHandle );
};

class IPCCopyDataServer : public IIPCServer
{
public:
	enum IPC_COPYDATA_COMMUNICATION_TYPE
	{
		IPC_COPYDATA_PACKET_RECEIVE = 0,
		IPC_COPYDATA_PACKET_CONNECT = 1,
		IPC_COPYDATA_PACKET_CLOSE = 2,
		IPC_COPYDATA_PACKET_START = 3,
		IPC_COPYDATA_PACKET_CLOSE_REPLAY = 4,
	};

	IPCCopyDataServer(IPCCopyDataServerEventHooker *hooker = NULL );
	virtual ~IPCCopyDataServer();
	bool Create( IIPCKey *host );
	BOOL IsExist( IIPCKey *client );
	IIPCHandle *Find( IIPCKey *client, IIPCReceiveCallback *callback = NULL );
	int Receive( IPC_COPYDATA_COMMUNICATION_TYPE wParam, COPYDATASTRUCT *data );
	IIPCHandleList GetHandleList();

	void SetHooker( IPCCopyDataServerEventHooker *hooker );

private:
	bool ProcessConnect( COPYDATASTRUCT *data );
	bool ProcessClose( COPYDATASTRUCT *data );
	bool ProcessReceive( COPYDATASTRUCT *data );
	bool ProcessReceiveStart( COPYDATASTRUCT *data );

private:
	IPCCopyDataHandle* SearchHandle(HWND hWnd);
	IPCCopyDataHandle* GetHandle(HWND hWnd, LPCTSTR name, IIPCReceiveCallback *callback );
	IPCCopyDataHandle* CreateNewHandle(HWND hWnd, LPCTSTR name);

private:
	typedef std::map< HWND, IPCCopyDataHandle* > IPCCopyDataHANDLE_MAP;
	IPCCopyDataHANDLE_MAP			m_handleMap;
	IPCCopyDataMessageWindow*		m_msgWnd;
	TCHAR							m_hostkey[IPC_COPYDATA_MAX_SEND_SIZE];
	IPCCopyDataServerEventHooker*	m_hooker;
};
};